@@ -48,12 +48,6 @@
  */
 
 /**
- * @mainpage
- * @include XI2proto.txt
- */
-
-/**
- * @file XI2proto.h
  * Protocol definitions for the XI2 protocol.
  * This file should not be included by clients that merely use XI2, but do not
  * need the wire protocol. Such clients should include XI2.h, or the matching
@@ -66,6 +60,7 @@
 #include <X11/Xproto.h>
 #include <X11/X.h>
 #include <X11/extensions/XI2.h>
+#include <stdint.h>
 
 /* make sure types have right sizes for protocol structures. */
 #define Window  uint32_t
@@ -75,8 +70,6 @@
 
 /**
  * XI2 Request opcodes
- * @addtogroup XI2Requests
- * @{
  */
 #define X_XIQueryPointer                40
 #define X_XIWarpPointer                 41
@@ -99,7 +92,6 @@
 #define X_XIDeleteProperty              58
 #define X_XIGetProperty                 59
 #define X_XIGetSelectedEvents           60
-/*@}*/
 
 /** Number of XI requests */
 #define XI2REQUESTS (X_XIGetSelectedEvents - X_XIQueryPointer + 1)
@@ -121,7 +113,6 @@ typedef struct {
 } FP3232;
 
 /**
- * \struct xXIDeviceInfo
  * Struct to describe a device.
  *
  * For a MasterPointer or a MasterKeyboard, 'attachment' specifies the
@@ -129,8 +120,6 @@ typedef struct {
  * For a SlaveKeyboard or SlavePointer, 'attachment' specifies the master
  * device this device is attached to.
  * For a FloatingSlave, 'attachment' is undefined.
- *
- * @see xXIQueryDeviceReq
  */
 typedef struct {
     uint16_t    deviceid;
@@ -145,13 +134,9 @@ typedef struct {
 } xXIDeviceInfo;
 
 /**
- * \struct xXIAnyInfo
  * Default template for a device class.
  * A device class is equivalent to a device's capabilities. Multiple classes
  * are supported per device.
- *
- * @see xXIQueryDeviceReq
- * @see xXIDeviceChangedEvent
  */
 typedef struct {
     uint16_t    type;           /**< One of *class */
@@ -164,9 +149,6 @@ typedef struct {
  * Denotes button capability on a device.
  * Struct is followed by num_buttons * Atom that names the buttons in the
  * device-native setup (i.e. ignoring button mappings).
- *
- * @see xXIQueryDeviceReq
- * @see xXIDeviceChangedEvent
  */
 typedef struct {
     uint16_t    type;           /**< Always ButtonClass */
@@ -179,9 +161,6 @@ typedef struct {
  * Denotes key capability on a device.
  * Struct is followed by num_keys * CARD32 that lists the keycodes available
  * on the device.
- *
- * @see xXIQueryDeviceReq
- * @see xXIDeviceChangedEvent
  */
 typedef struct {
     uint16_t    type;           /**< Always KeyClass */
@@ -193,9 +172,6 @@ typedef struct {
 /**
  * Denotes an valuator capability on a device.
  * One XIValuatorInfo describes exactly one valuator (axis) on the device.
- *
- * @see xXIQueryDevice
- * @see xXIDeviceChangedEvent
  */
 typedef struct {
     uint16_t    type;           /**< Always ValuatorClass       */
@@ -212,14 +188,27 @@ typedef struct {
     uint16_t    pad2;
 } xXIValuatorInfo;
 
+/***
+ * Denotes a scroll valuator on a device.
+ * One XIScrollInfo describes exactly one scroll valuator that must have a
+ * XIValuatorInfo struct.
+ */
+typedef struct {
+    uint16_t    type;           /**< Always ValuatorClass         */
+    uint16_t    length;         /**< Length in 4 byte units       */
+    uint16_t    sourceid;       /**< source device for this class */
+    uint16_t    number;         /**< Valuator number              */
+    uint16_t    scroll_type;    /**< ::XIScrollTypeVertical, ::XIScrollTypeHorizontal */
+    uint16_t    pad0;
+    uint32_t    flags;          /**< ::XIScrollFlagEmulate, ::XIScrollFlagPreferred   */
+    FP3232      increment;      /**< Increment for one unit of scrolling              */
+} xXIScrollInfo;
 
 /**
  * Used to select for events on a given window.
  * Struct is followed by (mask_len * CARD8), with each bit set representing
  * the event mask for the given type. A mask bit represents an event type if
  * (mask == (1 << type)).
- *
- * @see XISelectEvents
  */
 typedef struct {
     uint16_t    deviceid;       /**< Device id to select for        */
@@ -236,6 +225,7 @@ typedef struct
     uint32_t    base_mods;              /**< Logically pressed modifiers */
     uint32_t    latched_mods;           /**< Logically latched modifiers */
     uint32_t    locked_mods;            /**< Logically locked modifiers */
+    uint32_t    effective_mods;         /**< Effective modifiers */
 } xXIModifierInfo;
 
 /**
@@ -248,7 +238,7 @@ typedef struct
     uint8_t     base_group;             /**< Logically "pressed" group */
     uint8_t     latched_group;          /**< Logically latched group */
     uint8_t     locked_group;           /**< Logically locked group */
-    uint8_t     pad0;
+    uint8_t     effective_group;        /**< Effective group */
 } xXIGroupInfo;
 
 
@@ -259,7 +249,6 @@ typedef struct
  *************************************************************************************/
 
 /**
- * @struct xXIQueryVersionReq
  * Query the server for the supported X Input extension version.
  */
 
@@ -288,12 +277,9 @@ typedef struct {
 #define sz_xXIQueryVersionReply             32
 
 /**
- * @struct xXIQueryDeviceReq
  * Query the server for information about a specific device or all input
  * devices.
- *
  */
-
 typedef struct {
     uint8_t     reqType;                /**< Input extension major code */
     uint8_t     ReqType;                /**< Always ::X_XIQueryDevice */
@@ -319,7 +305,6 @@ typedef struct {
 #define sz_xXIQueryDeviceReply                  32
 
 /**
- * @struct xXISelectEventsReq
  * Select for events on a given window.
  */
 typedef struct {
@@ -333,7 +318,6 @@ typedef struct {
 #define sz_xXISelectEventsReq                  12
 
 /**
- * @struct xXIGetSelectedEventsReq
  * Query for selected events on a given window.
  */
 typedef struct {
@@ -361,7 +345,6 @@ typedef struct {
 #define sz_xXIGetSelectedEventsReply            32
 
 /**
- * @struct xXIQueryPointerReq
  * Query the given device's screen/window coordinates.
  */
 
@@ -393,10 +376,9 @@ typedef struct {
     xXIModifierInfo mods;
     xXIGroupInfo group;
 } xXIQueryPointerReply;
-#define sz_xXIQueryPointerReply                 52
+#define sz_xXIQueryPointerReply                 56
 
 /**
- * @struct xXIWarpPointerReq
  * Warp the given device's pointer to the specified position.
  */
 
@@ -406,19 +388,18 @@ typedef struct {
     uint16_t    length;                 /**< Length in 4 byte units */
     Window      src_win;
     Window      dst_win;
-    INT16       src_x;
-    INT16       src_y;
+    FP1616      src_x;
+    FP1616      src_y;
     uint16_t    src_width;
     uint16_t    src_height;
-    INT16       dst_x;
-    INT16       dst_y;
+    FP1616      dst_x;
+    FP1616      dst_y;
     uint16_t    deviceid;
     uint16_t    pad1;
 } xXIWarpPointerReq;
-#define sz_xXIWarpPointerReq                    28
+#define sz_xXIWarpPointerReq                    36
 
 /**
- * @struct xXIChangeCursorReq
  * Change the given device's sprite to the given cursor.
  */
 
@@ -434,7 +415,6 @@ typedef struct {
 #define sz_xXIChangeCursorReq                           16
 
 /**
- * @struct xXIChangeHierarchyReq
  * Modify the device hierarchy.
  */
 
@@ -505,7 +485,6 @@ typedef struct {
 
 
 /**
- * @struct xXISetClientPointerReq
  * Set the window/client's ClientPointer.
  */
 typedef struct {
@@ -519,7 +498,6 @@ typedef struct {
 #define sz_xXISetClientPointerReq                 12
 
 /**
- * @struct xXIGetClientPointerReq
  * Query the given window/client's ClientPointer setting.
  */
 typedef struct {
@@ -547,7 +525,6 @@ typedef struct {
 #define sz_xXIGetClientPointerReply               32
 
 /**
- * @struct xXISetFocusReq
  * Set the input focus to the specified window.
  */
 typedef struct {
@@ -562,7 +539,6 @@ typedef struct {
 #define sz_xXISetFocusReq                       16
 
 /**
- * @struct xXIGetDeviceFocusReq
  * Query the current input focus.
  */
 typedef struct {
@@ -590,7 +566,6 @@ typedef struct {
 
 
 /**
- * @struct xXIGrabDeviceReq
  * Grab the given device.
  */
 typedef struct {
@@ -636,7 +611,6 @@ typedef struct {
 #define sz_xXIGrabDeviceReply                  32
 
 /**
- * @struct xXIUngrabDeviceReq
  * Ungrab the specified device.
  *
  */
@@ -652,7 +626,6 @@ typedef struct {
 
 
 /**
- * @struct xXIAllowEventsReq
  * Allow or replay events on the specified grabbed device.
  */
 typedef struct {
@@ -668,7 +641,6 @@ typedef struct {
 
 
 /**
- * @struct xXIPassiveGrabDeviceReq
  * Passively grab the device.
  */
 typedef struct {
@@ -706,7 +678,6 @@ typedef struct {
 #define sz_xXIPassiveGrabDeviceReply            32
 
 /**
- * @struct xXIPassiveUngrabDeviceReq
  * Delete a passive grab for the given device.
  */
 typedef struct {
@@ -724,7 +695,6 @@ typedef struct {
 #define sz_xXIPassiveUngrabDeviceReq            20
 
 /**
- * @struct xXIListPropertiesReq
  * List all device properties on the specified device.
  */
 typedef struct {
@@ -752,7 +722,6 @@ typedef struct {
 #define sz_xXIListPropertiesReply               32
 
 /**
- * @struct xXIChangePropertyReq
  * Change a property on the specified device.
  */
 typedef struct {
@@ -769,7 +738,6 @@ typedef struct {
 #define sz_xXIChangePropertyReq                 20
 
 /**
- * @struct xXIDeletePropertyReq
  * Delete the specified property.
  */
 typedef struct {
@@ -783,7 +751,6 @@ typedef struct {
 #define sz_xXIDeletePropertyReq                 12
 
 /**
- * @struct xXIGetPropertyReq
  * Query the specified property's values.
  */
 typedef struct {
@@ -827,7 +794,6 @@ typedef struct {
  *************************************************************************************/
 
 /**
- * @struct xXIGenericDeviceEvent
  * Generic XI2 event header. All XI2 events use the same header.
  */
 typedef struct
@@ -841,12 +807,6 @@ typedef struct
     Time        time;
 } xXIGenericDeviceEvent;
 
-/**
- * @struct xXIHierarchyEvent
- * The device hierarchy has been modified. This event includes the device
- * hierarchy after the modification has been applied.
- */
-
 /**
  * Device hierarchy information.
  */
@@ -867,6 +827,10 @@ typedef struct
                                              ::XIDeviceEnabled, ::XIDeviceDisabled */
 } xXIHierarchyInfo;
 
+/**
+ * The device hierarchy has been modified. This event includes the device
+ * hierarchy after the modification has been applied.
+ */
 typedef struct
 {
     uint8_t     type;                   /**< Always GenericEvent */
@@ -887,7 +851,6 @@ typedef struct
 } xXIHierarchyEvent;
 
 /**
- * @struct xXIDeviceChangedEvent
  * A device has changed capabilities.
  */
 typedef struct
@@ -909,10 +872,8 @@ typedef struct
 } xXIDeviceChangedEvent;
 
 /**
- * @struct xXIDeviceEvent
  * Default input event for pointer or keyboard input.
  */
-
 typedef struct
 {
     uint8_t     type;                   /**< Always GenericEvent */
@@ -935,13 +896,13 @@ typedef struct
     uint16_t    valuators_len;          /**< Len of val. flags in 4 b units */
     uint16_t    sourceid;               /**< The source device */
     uint16_t    pad0;
+    uint32_t    flags;                  /**< ::XIKeyRepeat */
     xXIModifierInfo     mods;
     xXIGroupInfo        group;
 } xXIDeviceEvent;
 
 
 /**
- * @struct xXIRawEvent
  * Sent when an input event is generated. RawEvents include valuator
  * information in both device-specific data (i.e. unaccelerated) and
  * processed data (i.e. accelerated, if applicable).
@@ -956,17 +917,14 @@ typedef struct
     uint16_t    deviceid;
     Time        time;
     uint32_t    detail;
-    uint16_t    eventtype;              /**< ::XI_Motion, ::XI_ButtonPress,
-                                             ::XI_ButtonRelease, ::XI_KeyPress,
-                                             ::XI_KeyRelease */
+    uint16_t    sourceid;               /**< The source device (XI 2.1) */
     uint16_t    valuators_len;          /**< Length of trailing valuator
                                              mask in 4 byte units */
-    uint32_t    pad1;
+    uint32_t    flags;                  /**< ::XIKeyRepeat */
     uint32_t    pad2;
 } xXIRawEvent;
 
 /**
- * @struct xXIEnterEvent
  * Note that the layout of root, event, child, root_x, root_y, event_x,
  * event_y must be identical to the xXIDeviceEvent.
  */
@@ -1003,7 +961,6 @@ typedef xXIEnterEvent xXIFocusInEvent;
 typedef xXIEnterEvent xXIFocusOutEvent;
 
 /**
- * @struct xXIPropertyEvent
  * Sent when a device property is created, modified or deleted. Does not
  * include property data, the client is required to query the data.
  */
